// Copyright (C) 2018 Katayama Hirofumi MZ.
// This file is public domain software.
#include <windows.h>
#include <windowsx.h>
#include <shlobj.h>
#include <string>
#include <stdio.h>

void DoAddLine(HWND hwnd, const WCHAR *psz)
{
    const size_t c_count = 1024 * 32;
    static WCHAR szText[c_count];

    GetDlgItemText(hwnd, edt2, szText, c_count);

    std::wstring str = szText;
    str += psz;

    SetDlgItemText(hwnd, edt2, str.c_str());
}

void DoPrint(HWND hwnd, const WCHAR *pszFormat, ...)
{
    va_list va;
    va_start(va, pszFormat);
    WCHAR sz[MAX_PATH * 2];
    wvsprintfW(sz, pszFormat, va);
    DoAddLine(hwnd, sz);
    va_end(va);
}

BOOL DoTest(HWND hwnd, const WCHAR *pszLinkFile)
{
    HRESULT hr;
    IShellLink *pShellLink;
    IPersistFile *pPersistFile;
    IExtractIconW *pExtractIcon;
    WIN32_FIND_DATA find;
    BOOL bOK = FALSE;

    hr = CoCreateInstance(CLSID_ShellLink, NULL, CLSCTX_INPROC_SERVER, IID_IShellLink, (LPVOID *)&pShellLink);
    if (SUCCEEDED(hr))
    {
        hr = pShellLink->QueryInterface(IID_IPersistFile, (LPVOID *)&pPersistFile);
        if (SUCCEEDED(hr))
        {
            hr = pPersistFile->Load(pszLinkFile, STGM_READ);
            if (SUCCEEDED(hr))
            {
                WCHAR szPath[MAX_PATH], szIconPath[MAX_PATH];
                INT iIcon;

                hr = pShellLink->GetPath(szPath, MAX_PATH, &find, SLGP_SHORTPATH);
                DoPrint(hwnd, L"GetPath: %s (hr: 0x%08X)\r\n", szPath, hr);

                hr = pShellLink->GetIconLocation(szIconPath, MAX_PATH, &iIcon);
                DoPrint(hwnd, L"GetIconLocation: %s, %d (hr: 0x%08X)\r\n", szIconPath, iIcon, hr);

                hr = pShellLink->QueryInterface(IID_IExtractIconW, (LPVOID *)&pExtractIcon);
                if (SUCCEEDED(hr))
                {
                    HICON hIconLarge = NULL, hIconSmall = NULL;
                    hr = pExtractIcon->Extract(szIconPath, iIcon, &hIconLarge, &hIconSmall, MAKELONG(32, 16));

                    DoPrint(hwnd, L"Extract: %p, %p (hr: 0x%08X)\r\n", hIconLarge, hIconSmall, hr);

                    SendDlgItemMessage(hwnd, ico1, STM_SETICON, (WPARAM)hIconLarge, 0);

                    pExtractIcon->Release();
                }

                bOK = TRUE;
            }
            pPersistFile->Release();
        }
        pShellLink->Release();
    }

    return bOK;
}

void OnOK(HWND hwnd)
{
    WCHAR szText[MAX_PATH];
    GetDlgItemText(hwnd, edt1, szText, MAX_PATH);

    DoTest(hwnd, szText);
}

void OnCommand(HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
    switch (id)
    {
    case IDOK:
        OnOK(hwnd);
        break;
    case IDCANCEL:
        EndDialog(hwnd, IDCANCEL);
        break;
    }
}

BOOL OnInitDialog(HWND hwnd, HWND hwndFocus, LPARAM lParam)
{
    return TRUE;
}

INT_PTR CALLBACK
DialogProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
    switch (uMsg)
    {
        HANDLE_MSG(hwnd, WM_INITDIALOG, OnInitDialog);
        HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
    }
    return 0;
}

extern "C" INT WINAPI
wWinMain(HINSTANCE   hInstance,
         HINSTANCE   hPrevInstance,
         LPWSTR      lpCmdLine,
         INT         nCmdShow)
{
    CoInitialize(NULL);

    DialogBox(hInstance, MAKEINTRESOURCE(1), NULL, DialogProc);

    CoUninitialize();
    return 0;
}
